@@ -1303,6 +1303,21 @@ func TestWithTypeJSONOverrideLoader(t *testing.T) {
 	})
 }
 
+func TestWithTypeJSONOverrideLoaderGlob(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import("./foo" + bar, { with: { type: 'json' } }).then(console.log)
+			`,
+			"/foo.js": `{ "this is json not js": true }`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode: config.ModeBundle,
+		},
+	})
+}
+
 func TestWithBadType(t *testing.T) {
 	loader_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1614,6 +1629,55 @@ func TestLoaderBundleWithImportAttributes(t *testing.T) {
 	})
 }
 
+func TestLoaderBundleWithUnknownImportAttributesAndJSLoader(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import foo from "./foo.js" with { type: 'js' }
+				import bar from "./bar.js" with { js: 'true' }
+				import foo2 from "data:text/javascript,foo" with { type: 'js' }
+				import bar2 from "data:text/javascript,bar" with { js: 'true' }
+				console.log(foo, bar, foo2, bar2)
+			`,
+			"/foo.js": `...`,
+			"/bar.js": `,,,`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+		},
+		expectedScanLog: `entry.js: ERROR: Importing with a type attribute of "js" is not supported
+entry.js: ERROR: Importing with the "js" attribute is not supported
+entry.js: ERROR: Importing with a type attribute of "js" is not supported
+entry.js: ERROR: Importing with the "js" attribute is not supported
+`,
+	})
+}
+
+func TestLoaderBundleWithUnknownImportAttributesAndCopyLoader(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import foo from "./foo.thing" with { type: 'whatever' }
+				import bar from "./bar.thing" with { whatever: 'true' }
+				console.log(foo, bar)
+			`,
+			"/foo.thing": `...`,
+			"/bar.thing": `,,,`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode: config.ModeBundle,
+			ExtensionToLoader: map[string]config.Loader{
+				".js":    config.LoaderJS,
+				".thing": config.LoaderCopy,
+			},
+			AbsOutputFile: "/out.js",
+		},
+	})
+}
+
 func TestLoaderBundleWithTypeJSONOnlyDefaultExport(t *testing.T) {
 	loader_suite.expectBundled(t, bundled{
 		files: map[string]string{
@@ -1632,3 +1696,46 @@ func TestLoaderBundleWithTypeJSONOnlyDefaultExport(t *testing.T) {
 `,
 	})
 }
+
+func TestLoaderJSONPrototype(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import data from "./data.json"
+				console.log(data)
+			`,
+			"/data.json": `{
+				"": "The property below should be converted to a computed property:",
+				"__proto__": { "foo": "bar" }
+			}`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:          config.ModeBundle,
+			AbsOutputFile: "/out.js",
+			MinifySyntax:  true,
+		},
+	})
+}
+
+func TestLoaderJSONPrototypeES5(t *testing.T) {
+	loader_suite.expectBundled(t, bundled{
+		files: map[string]string{
+			"/entry.js": `
+				import data from "./data.json"
+				console.log(data)
+			`,
+			"/data.json": `{
+				"": "The property below should NOT be converted to a computed property for ES5:",
+				"__proto__": { "foo": "bar" }
+			}`,
+		},
+		entryPaths: []string{"/entry.js"},
+		options: config.Options{
+			Mode:                  config.ModeBundle,
+			AbsOutputFile:         "/out.js",
+			MinifySyntax:          true,
+			UnsupportedJSFeatures: es(5),
+		},
+	})
+}
