@@ -226,6 +226,13 @@ func parseOptionsImpl(
 				)
 			}
 
+		case strings.HasPrefix(arg, "--drop-labels="):
+			if buildOpts != nil {
+				buildOpts.DropLabels = splitWithEmptyCheck(arg[len("--drop-labels="):], ",")
+			} else {
+				transformOpts.DropLabels = splitWithEmptyCheck(arg[len("--drop-labels="):], ",")
+			}
+
 		case strings.HasPrefix(arg, "--legal-comments="):
 			value := arg[len("--legal-comments="):]
 			var legalComments api.LegalComments
@@ -414,8 +421,12 @@ func parseOptionsImpl(
 		case strings.HasPrefix(arg, "--tsconfig=") && buildOpts != nil:
 			buildOpts.Tsconfig = arg[len("--tsconfig="):]
 
-		case strings.HasPrefix(arg, "--tsconfig-raw=") && transformOpts != nil:
-			transformOpts.TsconfigRaw = arg[len("--tsconfig-raw="):]
+		case strings.HasPrefix(arg, "--tsconfig-raw="):
+			if buildOpts != nil {
+				buildOpts.TsconfigRaw = arg[len("--tsconfig-raw="):]
+			} else {
+				transformOpts.TsconfigRaw = arg[len("--tsconfig-raw="):]
+			}
 
 		case strings.HasPrefix(arg, "--entry-names=") && buildOpts != nil:
 			buildOpts.EntryNames = arg[len("--entry-names="):]
@@ -602,12 +613,15 @@ func parseOptionsImpl(
 		case strings.HasPrefix(arg, "--packages=") && buildOpts != nil:
 			value := arg[len("--packages="):]
 			var packages api.Packages
-			if value == "external" {
+			switch value {
+			case "bundle":
+				packages = api.PackagesBundle
+			case "external":
 				packages = api.PackagesExternal
-			} else {
+			default:
 				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
 					fmt.Sprintf("Invalid value %q in %q", value, arg),
-					"The only valid value is \"external\".",
+					"Valid values are \"bundle\" or \"external\".",
 				)
 			}
 			buildOpts.Packages = packages
@@ -740,6 +754,21 @@ func parseOptionsImpl(
 				transformOpts.LogLimit = limit
 			}
 
+		case strings.HasPrefix(arg, "--line-limit="):
+			value := arg[len("--line-limit="):]
+			limit, err := strconv.Atoi(value)
+			if err != nil || limit < 0 {
+				return parseOptionsExtras{}, cli_helpers.MakeErrorWithNote(
+					fmt.Sprintf("Invalid value %q in %q", value, arg),
+					"The line limit must be a non-negative integer.",
+				)
+			}
+			if buildOpts != nil {
+				buildOpts.LineLimit = limit
+			} else {
+				transformOpts.LineLimit = limit
+			}
+
 			// Make sure this stays in sync with "PrintErrorToStderr"
 		case isBoolFlag(arg, "--color"):
 			if value, err := parseBoolFlag(arg, true); err != nil {
@@ -816,6 +845,7 @@ func parseOptionsImpl(
 				"chunk-names":        true,
 				"color":              true,
 				"conditions":         true,
+				"drop-labels":        true,
 				"entry-names":        true,
 				"footer":             true,
 				"format":             true,
@@ -849,6 +879,7 @@ func parseOptionsImpl(
 				"public-path":        true,
 				"reserve-props":      true,
 				"resolve-extensions": true,
+				"serve-fallback":     true,
 				"serve":              true,
 				"servedir":           true,
 				"source-root":        true,
@@ -952,6 +983,8 @@ func parseTargets(targets []string, arg string) (target api.Target, engines []ap
 		"es2020": api.ES2020,
 		"es2021": api.ES2021,
 		"es2022": api.ES2022,
+		"es2023": api.ES2023,
+		"es2024": api.ES2024,
 	}
 
 outer:
@@ -1049,37 +1082,80 @@ func splitWithEmptyCheck(s string, sep string) []string {
 	return strings.Split(s, sep)
 }
 
-func runImpl(osArgs []string) int {
-	analyze := false
-	analyzeVerbose := false
-	end := 0
+type analyzeMode uint8
 
+const (
+	analyzeDisabled analyzeMode = iota
+	analyzeEnabled
+	analyzeVerbose
+)
+
+func filterAnalyzeFlags(osArgs []string) ([]string, analyzeMode) {
+	analyze := analyzeDisabled
+	end := 0
 	for _, arg := range osArgs {
-		// Special-case running a server
-		if arg == "--serve" || strings.HasPrefix(arg, "--serve=") || strings.HasPrefix(arg, "--servedir=") {
+		switch arg {
+		case "--analyze":
+			analyze = analyzeEnabled
+		case "--analyze=verbose":
+			analyze = analyzeVerbose
+		default:
+			osArgs[end] = arg
+			end++
+		}
+	}
+	return osArgs[:end], analyze
+}
+
+// Print metafile analysis after the build if it's enabled
+func addAnalyzePlugin(buildOptions *api.BuildOptions, analyze analyzeMode, osArgs []string) {
+	buildOptions.Plugins = append(buildOptions.Plugins, api.Plugin{
+		Name: "PrintAnalysis",
+		Setup: func(build api.PluginBuild) {
+			color := logger.OutputOptionsForArgs(osArgs).Color
+			build.OnEnd(func(result *api.BuildResult) (api.OnEndResult, error) {
+				if result.Metafile != "" {
+					logger.PrintTextWithColor(os.Stderr, color, func(colors logger.Colors) string {
+						return api.AnalyzeMetafile(result.Metafile, api.AnalyzeMetafileOptions{
+							Color:   colors != logger.Colors{},
+							Verbose: analyze == analyzeVerbose,
+						})
+					})
+					os.Stderr.WriteString("\n")
+				}
+				return api.OnEndResult{}, nil
+			})
+		},
+	})
+
+	// Always generate a metafile if we're analyzing, even if it won't be written out
+	buildOptions.Metafile = true
+}
+
+func runImpl(osArgs []string, plugins []api.Plugin) int {
+	// Special-case running a server
+	for _, arg := range osArgs {
+		if arg == "--serve" ||
+			strings.HasPrefix(arg, "--serve=") ||
+			strings.HasPrefix(arg, "--servedir=") ||
+			strings.HasPrefix(arg, "--serve-fallback=") {
 			serveImpl(osArgs)
 			return 1 // There was an error starting the server if we get here
 		}
-
-		// Special-case analyze just for our CLI
-		if arg == "--analyze" {
-			analyze = true
-			analyzeVerbose = false
-			continue
-		}
-		if arg == "--analyze=verbose" {
-			analyze = true
-			analyzeVerbose = true
-			continue
-		}
-
-		osArgs[end] = arg
-		end++
 	}
-	osArgs = osArgs[:end]
 
+	osArgs, analyze := filterAnalyzeFlags(osArgs)
 	buildOptions, transformOptions, extras, err := parseOptionsForRun(osArgs)
 
+	// Add any plugins from the caller after parsing the build options
+	if buildOptions != nil {
+		buildOptions.Plugins = append(buildOptions.Plugins, plugins...)
+	}
+
+	if analyze != analyzeDisabled {
+		addAnalyzePlugin(buildOptions, analyze, osArgs)
+	}
+
 	switch {
 	case buildOptions != nil:
 		// Read the "NODE_PATH" from the environment. This is part of node's
@@ -1158,7 +1234,7 @@ func runImpl(osArgs []string) int {
 					logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
 						"Failed to create output directory: %s", err.Error()))
 				} else {
-					if err := ioutil.WriteFile(metafileAbsPath, []byte(json), 0644); err != nil {
+					if err := ioutil.WriteFile(metafileAbsPath, []byte(json), 0666); err != nil {
 						logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
 							"Failed to write to output file: %s", err.Error()))
 					}
@@ -1205,7 +1281,7 @@ func runImpl(osArgs []string) int {
 						"Failed to create output directory: %s", err.Error()))
 				} else {
 					bytes := printMangleCache(mangleCache, mangleCacheOrder, buildOptions.Charset == api.CharsetASCII)
-					if err := ioutil.WriteFile(mangleCacheAbsPath, bytes, 0644); err != nil {
+					if err := ioutil.WriteFile(mangleCacheAbsPath, bytes, 0666); err != nil {
 						logger.PrintErrorToStderr(osArgs, fmt.Sprintf(
 							"Failed to write to output file: %s", err.Error()))
 					}
@@ -1213,36 +1289,11 @@ func runImpl(osArgs []string) int {
 			}
 		}
 
-		// Print metafile analysis after the build if it's enabled
-		var printAnalysis func(metafile string)
-		if analyze {
-			printAnalysis = func(metafile string) {
-				if metafile == "" {
-					return
-				}
-				logger.PrintTextWithColor(os.Stderr, logger.OutputOptionsForArgs(osArgs).Color, func(colors logger.Colors) string {
-					return api.AnalyzeMetafile(metafile, api.AnalyzeMetafileOptions{
-						Color:   colors != logger.Colors{},
-						Verbose: analyzeVerbose,
-					})
-				})
-				os.Stderr.WriteString("\n")
-			}
-
-			// Always generate a metafile if we're analyzing, even if it won't be written out
-			buildOptions.Metafile = true
-		}
-
 		// Handle post-build actions with a plugin so they also work in watch mode
 		buildOptions.Plugins = append(buildOptions.Plugins, api.Plugin{
 			Name: "PostBuildActions",
 			Setup: func(build api.PluginBuild) {
 				build.OnEnd(func(result *api.BuildResult) (api.OnEndResult, error) {
-					// Print our analysis of the metafile
-					if printAnalysis != nil {
-						printAnalysis(result.Metafile)
-					}
-
 					// Write the metafile to the file system
 					if writeMetafile != nil {
 						writeMetafile(result.Metafile)
@@ -1313,6 +1364,7 @@ func parseServeOptionsImpl(osArgs []string) (api.ServeOptions, []string, error)
 	servedir := ""
 	keyfile := ""
 	certfile := ""
+	fallback := ""
 
 	// Filter out server-specific flags
 	filteredArgs := make([]string, 0, len(osArgs))
@@ -1327,6 +1379,8 @@ func parseServeOptionsImpl(osArgs []string) (api.ServeOptions, []string, error)
 			keyfile = arg[len("--keyfile="):]
 		} else if strings.HasPrefix(arg, "--certfile=") {
 			certfile = arg[len("--certfile="):]
+		} else if strings.HasPrefix(arg, "--serve-fallback=") {
+			fallback = arg[len("--serve-fallback="):]
 		} else {
 			filteredArgs = append(filteredArgs, arg)
 		}
@@ -1356,6 +1410,7 @@ func parseServeOptionsImpl(osArgs []string) (api.ServeOptions, []string, error)
 		Servedir: servedir,
 		Keyfile:  keyfile,
 		Certfile: certfile,
+		Fallback: fallback,
 	}, filteredArgs, nil
 }
 
@@ -1372,11 +1427,15 @@ func serveImpl(osArgs []string) {
 	options.LogLimit = 5
 	options.LogLevel = api.LogLevelInfo
 
+	filteredArgs, analyze := filterAnalyzeFlags(filteredArgs)
 	extras, errWithNote := parseOptionsImpl(filteredArgs, &options, nil, kindInternal)
 	if errWithNote != nil {
 		logger.PrintErrorWithNoteToStderr(osArgs, errWithNote.Text, errWithNote.Note)
 		return
 	}
+	if analyze != analyzeDisabled {
+		addAnalyzePlugin(&options, analyze, osArgs)
+	}
 
 	serveOptions.OnRequest = func(args api.ServeOnRequestArgs) {
 		logger.PrintText(os.Stderr, logger.LevelInfo, filteredArgs, func(colors logger.Colors) string {
